local SR_Cooldowns = SpellReminder:NewModule("SR_Cooldowns","AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SpellReminder")

local function ClipNumber(number, minimum, maximum, trueMax)
	return min(max(min(number,maximum),minimum),trueMax)
end

local defaults = {
	profile = {
		enabled = false,
	},
}
local options = {
	type = "group",
	name = L["Cooldowns"],
	order = 600,
	childGroups = "tab",
	args = {
		enabled = {
			type = "toggle",
			name = L["Enabled"],
			desc = L["Enabled"],
			get = function()
				return SR_Cooldowns:IsEnabled()
			end,
			set = function(info, v)
				if v then
					SR_Cooldowns:Enable()
				else
					SR_Cooldowns:Disable()
				end
				SR_Cooldowns.db.profile.enabled = v
			end,
			order = 100,
		},
	},
}

function SR_Cooldowns:OnInitialize()
	self.db = SpellReminder.db:RegisterNamespace('Cooldowns', defaults)
	db = self.db.profile
	
	if SpellReminder.db.profile.docooldowns then
		self.db.profile.enabled = SpellReminder.db.profile.docooldowns
		SpellReminder.db.profile.docooldowns = nil
	end
	
	self:SetEnabledState(self.db.profile.enabled)
	SpellReminder:RegisterModuleOptions('Cooldowns', options, L["Cooldowns"])
end

function SR_Cooldowns:OnDisable()
	self:UnregisterAllEvents()
	for eventName,info in pairs(SpellReminder.track.active) do 
		if info.tGUID == "COOLDOWN" then
			SpellReminder:CleanUp(eventName)
		end
	end
end

function SR_Cooldowns:OnEnable()
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN","ProcessCooldowns")
	self:RegisterEvent("PLAYER_ENTERING_WORLD","ProcessCooldowns")
	self:ProcessCooldowns()
end

function SR_Cooldowns:ProcessCooldowns()
--	for eventName,info in pairs(SpellReminder.track.active) do 
--		if info.tGUID == "COOLDOWN" then
--			SpellReminder:CleanUp(eventName)
--		end
--	end
	
	local i = 1
	local cooldowns,done = {},{}
	local rangedSpeed = UnitRangedDamage("player");
	while true do
		local spellName, spellRank = GetSpellBookItemName(i, BOOKTYPE_SPELL)
		if (not spellName) then	break end
		local start, duration, enabled = GetSpellCooldown(i,BOOKTYPE_SPELL);
		if (not done[spellName]) and enabled == 1 and duration > 1.5 and duration > rangedSpeed then
			done[spellName] = true
			local tabName, tabTexture
			local useTexture = GetSpellTexture(i,BOOKTYPE_SPELL)
			for t = 1, MAX_SKILLLINE_TABS do -- find the spell tree
				local name, texture, offset, numSpells = GetSpellTabInfo(t)
				if not name then break end
				if (i > offset) and (i < offset + numSpells + 1) then
					tabName = name
					tabTexture = texture
					break
				end
			end
			local index = floor(start*duration)..(tabName or "")
			local eventName = SpellReminder:GetEventName(index,"COOLDOWN")
			cooldowns[index] = (cooldowns[index] or 0) + 1  --- multiple spells with the same name ????  PW:Shield
			if cooldowns[index] > 1 then
				if not tabName then SpellReminder:Print("no tab for "..spellName) end
				spellName = (tabName or "").. " Tree"
				useTexture = tabTexture
--				spellName = "Multiple Spells"
			end
			
			local remaining = start + duration - GetTime()
			local setupID = 'cd:'..spellName
			if not SpellReminder.chardb.char.spellSetup[setupID].installed then
				SpellReminder:Print("Detected new cooldown! ("..spellName..")")
				SpellReminder.chardb.char.spellSetup[setupID] = SpellReminder:InstallWatch(setupID,spellRank,nil,duration,useTexture)
			end
			local setupInfo = SpellReminder.chardb.char.spellSetup[setupID]
			SpellReminder:SetSpellCategory(setupID,'Cooldowns')-- ('..tabName..')')
			if setupInfo.installed and setupInfo.active then
				local color = (setupInfo.installed and setupInfo.c) or nil
				local ci = SpellReminder:GetCastingObject(setupID,setupInfo.displayName,spellRank,"Cooldown","COOLDOWN",start,duration,ClipNumber(duration*0.125,5,15,duration),eventName,color,"cooldown",setupInfo.icon)
				if cooldowns[index] > 1 then ci.applications = cooldowns[index] end
				ci.customFormat = "(CD%a) %n"
				self:TrackSpell(ci,true)
			end
		else
			-- clear bar
			SpellReminder:CleanUp(eventName)
		end
		i = i + 1
	end
	done = nil
	cooldowns = nil
end